//-----------------------------------------------------------------------------
// UTF8 converter
//-----------------------------------------------------------------------------
var UTF8 = {
  encode: function(string) {
    var utftext = "";
    var n = 0;
    var k = 0;
    var c = 0;
    string = string.replace(/\r\n/g, "\n");
    for (n = 0, k = string.length; n < k; n++) {
      c = string.charCodeAt(n);
      if (c < 128) {
        utftext += String.fromCharCode(c);
      }
      else if ((c > 127) && (c < 2048)) {
        utftext += String.fromCharCode((c >> 6) | 192);
        utftext += String.fromCharCode((c & 63) | 128);
      }
      else {
        utftext += String.fromCharCode((c >> 12) | 224);
        utftext += String.fromCharCode(((c >> 6) & 63) | 128);
        utftext += String.fromCharCode((c & 63) | 128);
      }
    }
    return utftext;
  },
  decode: function(utftext) {
    var string = "";
    var i = 0;
    var c = c1 = c2 = 0;
    while (i < utftext.length) {
      c = utftext.charCodeAt(i);
      if (c < 128) {
        string += String.fromCharCode(c);
        i++;
      }
      else if ((c > 191) && (c < 224)) {
        c2 = utftext.charCodeAt(i + 1);
        string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
        i += 2;
      }
      else {
        c2 = utftext.charCodeAt(i + 1);
        c3 = utftext.charCodeAt(i + 2);
        string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
        i += 3;
      }
    }
    return string;
  },
  needEncoding: function(text) {
    var encode = false;
    var i = 0;
    // Text is either latin-1 or utf-8 encoded, let's assume utf8 if text contain
    // C3h followed by a character with 0x80 set. C3h is  in latin-1, so we may
    // fail, but not likely...
    for (i = 0; i < text.length; i++) {
      if (((i + 1) < text.length) && (text.charCodeAt(i) == 0xC3)) {
        // Text have utf8 start character
        encode = false;
        i++;
        if ((text.charCodeAt(i) & 0x80) == 0) {
          // Not two bytes, so text contain latin1 
          encode = true;
          break;
        }
      }
      else if ((text.charCodeAt(i) & 0x80) != 0) {
        // Text contain non-ascii character without utf8 start character
        encode = true;
        break;
      }
    }
    return encode;
  }
}
//-----------------------------------------------------------------------------
// BASE64 converter
//-----------------------------------------------------------------------------
var B64_STR = 
    "ABCDEFGHIJKLMNOP" +
    "QRSTUVWXYZabcdef" +
    "ghijklmnopqrstuv" +
    "wxyz0123456789+/" +
    "=";
var BASE64 = {
  encode: function(input) {
    var output = "";
    var chr1, chr2, chr3 = "";
    var enc1, enc2, enc3, enc4 = "";
    var i = 0;
    do {
      chr1 = input.charCodeAt(i++);
      chr2 = input.charCodeAt(i++);
      chr3 = input.charCodeAt(i++);
      enc1 = chr1 >> 2;
      enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
      enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
      enc4 = chr3 & 63;
      if (isNaN(chr2)) {
        enc3 = enc4 = 64;
      }
      else if (isNaN(chr3)) {
        enc4 = 64;
      }
      output +=
          B64_STR.charAt(enc1) +
          B64_STR.charAt(enc2) +
          B64_STR.charAt(enc3) +
          B64_STR.charAt(enc4);
      chr1 = chr2 = chr3 = "";
      enc1 = enc2 = enc3 = enc4 = "";
    }
    while (i < input.length);
    return output;
  },
  decode: function(input) {
    var output = "";
    var chr1, chr2, chr3 = "";
    var enc1, enc2, enc3, enc4 = "";
    var i = 0;
    do {
      enc1 = B64_STR.indexOf(input.charAt(i++));
      enc2 = B64_STR.indexOf(input.charAt(i++));
      enc3 = B64_STR.indexOf(input.charAt(i++));
      enc4 = B64_STR.indexOf(input.charAt(i++));
      chr1 = (enc1 << 2) | (enc2 >> 4);
      chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
      chr3 = ((enc3 & 3) << 6) | enc4;
      output += String.fromCharCode(chr1);
      if (enc3 != 64)
        output += String.fromCharCode(chr2);
      if (enc4 != 64)
        output += String.fromCharCode(chr3);
      chr1 = chr2 = chr3 = "";
      enc1 = enc2 = enc3 = enc4 = "";
    } while (i < input.length);
    return output;
  }
}
//-----------------------------------------------------------------------------
// HEXADECIMAL converter
//-----------------------------------------------------------------------------
var HEX_STR = "0123456789ABCDEF";
var HEX = {
  encode: function(input) {
    var value = "";
    var i = 0;
    var c = 0;
    var c1 = 0;
    var c2 = 0;
    for (i = 0; i < input.length; i += 1) {
      c = input.charCodeAt(i);
      c1 = (c >> 4) & 0x0f;
      c2 = (c) & 0x0f;
      value += HEX_STR.charAt(c1);
      value += HEX_STR.charAt(c2);
    }
    return value;
  },
  decode: function(input) {
    var value = "";
    var i = 0;
    var c1 = 0;
    var c2 = 0;
    input = input.toUpperCase();
    for (i = 0; i < input.length; i += 2) {
      c1 = HEX_STR.indexOf(input.charAt(i));
      c2 = HEX_STR.indexOf(input.charAt(i + 1));
      value += String.fromCharCode((c1 * 16) + c2);
    }
    return value;
  }
}
//-----------------------------------------------------------------------------
// PEM converter
//-----------------------------------------------------------------------------
var PEM = {
  encode: function(name, data) {
    var value = "";
    var i = 0;
    var n = 0;
    value += "-----BEGIN " + name + "-----\r\n";
    while (i < data.length) {
      if ((i + 64) > data.length)
        n = data.length - i;
      else
        n = 64;
      value += data.substr(i, n) + "\r\n";
      i += n;
    }
    value += "-----END " + name + "-----\r\n";
    return value;
  },
  decode: function(value) {
    var data = "";
    var i = 0;
    var j = 0;
    if ((i = value.indexOf("BEGIN")) != -1) {
      if ((i = value.indexOf("-----", i)) != -1) {
        i += 5;
        if ((j = value.indexOf("-----", i)) != -1) {
          data = value.substr(i, j - i);
          data = data.replace(/\n/g, '');  
          data = data.replace(/\s/g, '').replace(/, /g, '');  
        }
      }
    }
    return data;
  }
}
//-----------------------------------------------------------------------------
// ConvertHtml
//-----------------------------------------------------------------------------
function ConvertHtml(input) {
  var i = 0;
  var c = 0;
  var c2 = 0;
  var output = "";
  for (i = 0; i < input.length; i++) {
    c = input.charCodeAt(i);
    // utf-8 (2-char)
    if ((c == 0xC3) && ((i+1) < input.length)) {
      c2 = input.charCodeAt(i+1);
      if (c2 == 0x84) 
        output += "&Auml;";
      else if (c2 == 0x85)
        output += "&Aring;";
      else if (c2 == 0x96)
        output += "&Ouml;";
      else if (c2 == 0x98)
        output += "&Oslash;";
      else if (c2 == 0x9C)
        output += "&Uuml;";
      else if (c2 == 0xA1)
        output += "&aacute;";
      else if (c2 == 0xA4)
        output += "&auml;";
      else if (c2 == 0xA5)
        output += "&aring;";
      else if (c2 == 0xA7)
        output += "&ccedil;";
      else if (c2 == 0xA9)
        output += "&eacute;";
      else if (c2 == 0xAA)
        output += "&ecirc;";
      else if (c2 == 0xB6)
        output += "&ouml;";
      else if (c2 == 0xB8)
        output += "&oslash;";
      else if (c2 == 0xBC)
        output += "&uuml;";
      else
        output += String.fromCharCode(c) + String.fromCharCode(c2);
      i++;
    }
    // latin-1 (1-char)
    else {
      if (c == 0xC4) 
        output += "&Auml;";
      else if (c == 0xC5)
        output += "&Aring;";
      else if (c == 0xD6)
        output += "&Ouml;";
      else if (c == 0xD8)
        output += "&Oslash;";
      else if (c == 0xDC)
        output += "&Uuml;";
      else if (c == 0xE1)
        output += "&aacute;";
      else if (c == 0xE4)
        output += "&auml;";
      else if (c == 0xE5)
        output += "&aring;";
      else if (c == 0xE7)
        output += "&ccedil;";
      else if (c == 0xE9)
        output += "&eacute;";
      else if (c == 0xEA)
        output += "&ecirc;";
      else if (c == 0xF6)
        output += "&ouml;";
      else if (c == 0xF8)
        output += "&oslash;";
      else if (c == 0xFC)
        output += "&uuml;";
      else
        output += String.fromCharCode(c);
    }
  }
  return output;
}
